/*
 * HealthcardIO - a library providing functionality for accessing healthcards
 *
 * Copyright (C) 2017  gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.api;

import de.gematik.security.iso7816.Iso7816UnsuccessfulExecutionException;

import javax.smartcardio.CardException;

/**
 * @author André Szdzuy
 */
public interface GematikCommands {

    /**
     * @return true if card is EGK, false if not
     * @throws Iso7816UnsuccessfulExecutionException if the card commands return an error
     *         (SW is stored in detailed message)
     * @throws CardException any Card IO exception
     */
    boolean isEGK() throws Iso7816UnsuccessfulExecutionException, CardException;

    /**
     * @return true if card is HBA, false if not
     * @throws Iso7816UnsuccessfulExecutionException if the card commands return an error
     *         (SW is stored in detailed message)
     * @throws CardException any Card IO exception
     */
    boolean isHBA() throws Iso7816UnsuccessfulExecutionException, CardException;

    /**
     * @return true if card is SMC-B, false if not
     * @throws Iso7816UnsuccessfulExecutionException if the card commands return an error
     *         (SW is stored in detailed message)
     * @throws CardException any Card IO exception
     */
    boolean isSMCB() throws Iso7816UnsuccessfulExecutionException, CardException;

    /**
     * @return a String representation of the card's type (EGK, HBA or SMC-B)
     * @throws Iso7816UnsuccessfulExecutionException if the card commands return an error
     *         (SW is stored in detailed message)
     * @throws CardException any Card IO exception
     */
    String getType() throws Iso7816UnsuccessfulExecutionException, CardException;

    /**
     * Reads the ICCSN value from file EF.GDO.
     *
     * @return the 12-byte ICCSN value
     * @throws Iso7816UnsuccessfulExecutionException if the card commands return an error
     *         (SW is stored in detailed message)
     * @throws CardException any Card IO exception
     */
    byte[] readICCSN() throws Iso7816UnsuccessfulExecutionException, CardException;

}
