/*
 * HealthcardIO - a library providing functionality for accessing healthcards
 *
 * Copyright (C) 2017  gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.crypto;

import de.gematik.security.impl.HealthcardContext;

import java.math.BigInteger;
import java.security.interfaces.RSAPrivateKey;

/**
 * This class represents a RSAPrivateKey that is stored on a smartcard.
 * It stores only the reference of the real key which is only stored at the card.
 *
 * @author André Szdzuy
 */
public class CardBasedRSAPrivateKey implements RSAPrivateKey {

    private int dfSpecificPrivateKeyRef;
    private HealthcardContext card;

    public CardBasedRSAPrivateKey(HealthcardContext card, int dfSpecificPrivateKeyRef) {
        this.card = card;
        this.dfSpecificPrivateKeyRef = dfSpecificPrivateKeyRef;
    }

    public int getDfSpecificPrivateKeyRef() {
        return dfSpecificPrivateKeyRef;
    }

    public void setDfSpecificPrivateKeyRef(int dfSpecificPrivateKeyRef) {
        this.dfSpecificPrivateKeyRef = dfSpecificPrivateKeyRef;
    }

    public HealthcardContext getCard() {
        return card;
    }

    public void setCard(HealthcardContext card) {
        this.card = card;
    }

    @Override
    public BigInteger getPrivateExponent() {
        throw new UnsupportedOperationException();
    }

    @Override
    public String getAlgorithm() {
        return "RSA";
    }

    @Override
    public String getFormat() {
        return null;
    }

    @Override
    public byte[] getEncoded() {
        return null;
    }

    @Override
    public BigInteger getModulus() {
        throw new UnsupportedOperationException();
    }
}
