/*
 * HealthcardIO - a library providing functionality for accessing healthcards
 *
 * Copyright (C) 2017  gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.iso7816;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.LinkedHashMap;

/**
 * @author Ronald Koenig
 */
public class TlvData {
    private static final Log LOG = LogFactory.getLog(TlvData.class);

    private LinkedHashMap<Integer, byte[]> linkedHashMap = new LinkedHashMap<>();

    public TlvData(){}

    public TlvData(byte[] data){
        int index=0;
        while(index<data.length) {
            try {
                //tag
                int tag = 0;
                if (data[index] == 0x5f || data[index] == 0x7f) {
                    tag = data[index++] << 8;
                }
                tag += data[index++]&0xff;
                //length
                int length = 0;
                if (data[index] == 0x82) {
                    length = (data[index++]&0xff) * 256;
                }
                length += data[index++]&0xff;
                byte[] bytes = new byte[length];
                System.arraycopy(data, index, bytes, 0, length);
                linkedHashMap.put(tag, bytes);
                index+=length;
//                LOG.info("new TlvData: tag = " + Utilities.toHexString(tag) + ", length = " + Utilities.toHexString(length));
            }catch(Exception e){
                LOG.error("malformed apdudata");
                break;
            }
        }
    }

    public TlvData putData(int tag, TlvData tlvData){
        return putData(tag, tlvData.getBytes());
    }

    public TlvData putData(int tag, byte[] data){
        if(tag>=0x10000){
            LOG.error("unsupported tag");
            return this;
        }
        if(data.length>=0x10000) {
            LOG.error("length >= 65536 isn't supported");
            return this;
        }
        linkedHashMap.put(tag, data);
        return this;
    }

    public byte[] getData(int tag){
        return linkedHashMap.get(tag);
    }

    public byte[] getBytes(){
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        for(int tag: linkedHashMap.keySet()){
            if(tag>255){
                bos.write((byte)((tag&0xff00)>>8));
            }
            bos.write((byte)(tag&0xFF));
            byte[] data = linkedHashMap.get(tag);
            if(data.length>255){
                bos.write((byte)0x82);
                bos.write((byte)((data.length&0xff00)>>8));
            }
            bos.write((byte)(data.length&0xff));
            try{
                bos.write(data);
            }catch(IOException e){
                LOG.error("encoding failure");
            }
       }
       return bos.toByteArray();
    }
}
