/*
 * HealthcardIO - a library providing functionality for accessing healthcards
 *
 * Copyright (C) 2017  gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.iso9564;

import de.gematik.security.Utilities.Utilities;

/**
 * A PIN bytes of format 2 according to ISO9564
 *
 * @author Ronald Koenig
 */
public class PinBlock {

    private static final String TAG = PinBlock.class.getName();

    private byte[] bytes;

    /**
     * Constructs a new PinBlock object.
     * <p>
     * @param pin as integer string
     */
    public PinBlock(String pin) throws PinBlockException {
        if(pin.matches("[0-9]{4,12}")){     // see COS spec chapter 8.1.7
            StringBuffer buffer = new StringBuffer();
            buffer.append("2");
            buffer.append(String.format("%X", pin.length()));
            buffer.append(pin);
            buffer.append("FFFFFFFFFFFFFFFF");
            bytes = Utilities.HexStringToByteArray(buffer.substring(0,16));
        } else {
            throw new PinBlockException("PIN must be numeric and must have length between 4 and 12.");
        }
    }

    public byte[] getBytes(){
        return bytes;
    }
}
