/*
 * HealthcardIO - a library providing functionality for accessing healthcards
 *
 * Copyright (C) 2017  gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.tr03110;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.spongycastle.asn1.ASN1Integer;
import org.spongycastle.asn1.ASN1ObjectIdentifier;
import org.spongycastle.asn1.ASN1SequenceParser;
import org.spongycastle.asn1.DLSequence;

import java.io.IOException;

/**
 * @author Ronald Koenig
 */
public class PaceInfo {
    private static final Log LOG = LogFactory.getLog(PaceInfo.class);

    //EGK protocols
    public final static String ID_PACE_ECDH_GM_AES_CBC_CMAC_128 = "0.4.0.127.0.7.2.2.4.2.2";
    public final static String ID_PACE_ECDH_GM_AES_CBC_CMAC_192 = "0.4.0.127.0.7.2.2.4.2.3";
    public final static String ID_PACE_ECDH_GM_AES_CBC_CMAC_256 = "0.4.0.127.0.7.2.2.4.2.4";
    public final static byte[] BYTES_PACE_ECDH_GM_AES_CBC_CMAC_128 = new byte[]{4, 0, 127, 0, 7, 2, 2, 4, 2, 2};
    public final static byte[] BYTES_PACE_ECDH_GM_AES_CBC_CMAC_192 = new byte[]{4, 0, 127, 0, 7, 2, 2, 4, 2, 3};
    public final static byte[] BYTES_PACE_ECDH_GM_AES_CBC_CMAC_256 = new byte[]{4, 0, 127, 0, 7, 2, 2, 4, 2, 4};


    private byte[] bytes = null;
    private String string = null;

    private ASN1ObjectIdentifier protocol;
    private int version;

    private int parameterID;

    public PaceInfo(DLSequence dlSequence) throws IOException{
        ASN1SequenceParser parser = dlSequence.parser();
        protocol = (ASN1ObjectIdentifier)parser.readObject();
        version = ((ASN1Integer)parser.readObject()).getValue().intValue();
        try {
            parameterID = ((ASN1Integer) parser.readObject()).getValue().intValue();
        }catch(NullPointerException e){
            // optional parameter
        }
    }

    public byte[] getProtocolBytes() {
        byte[] bytes = null;
        try {
            bytes = new byte[protocol.getEncoded().length - 2];
            System.arraycopy(protocol.getEncoded(), 2, bytes, 0, bytes.length);
        }catch(IOException e){
            LOG.error("encoding exception");
        }
        return bytes;
    }

    public String getProtocolID(){
        return protocol.getId();
    }

    public int getVersion() {return version;}

    public int getParameterID() {return parameterID;}
    public String getParameterIDString(){
        // standardized domain parameters used by EGK
        String parameter = null;
        switch(parameterID){
            case 13:
                parameter = "BrainpoolP256r1";
                break;
            case 16:
                parameter = "BrainpoolP384r1";
                break;
            case 17:
                parameter = "BrainpoolP512r1";
                break;
        }
        return parameter;
    }

}
